# GENERATED FILE #
from enum import Enum

from applitools.common.utils.general_utils import DeprecatedEnumVariant

__all__ = ("DynamicTextType",)


class DynamicTextType(Enum):
    TextField = "TextField"
    Number = "Number"
    Date = "Date"
    Link = "Link"
    Email = "Email"
    Currency = "Currency"
