# GENERATED FILE #
from enum import Enum

from applitools.common.utils.general_utils import DeprecatedEnumVariant

__all__ = ("DeviceName",)


class DeviceName(Enum):
    iPhone_4 = "iPhone 4"
    iPhone_SE_3rd_Gen = "iPhone SE 3rd Gen"
    iPhone_6_7_8 = "iPhone 6/7/8"
    iPhone_6_7_8_Plus = "iPhone 6/7/8 Plus"
    iPhone_X = "iPhone X"
    iPhone_13_Mini = "iPhone 13 Mini"
    iPhone_14_Plus = "iPhone 14 Plus"
    iPhone_14_Pro = "iPhone 14 Pro"
    iPhone_15_Pro = "iPhone 15 Pro"
    BlackBerry_Z30 = "BlackBerry Z30"
    Nexus_4 = "Nexus 4"
    Nexus_5 = "Nexus 5"
    Nexus_5X = "Nexus 5X"
    Nexus_6 = "Nexus 6"
    Nexus_6P = "Nexus 6P"
    Pixel_2 = "Pixel 2"
    Pixel_2_XL = "Pixel 2 XL"
    LG_Optimus_L70 = "LG Optimus L70"
    Nokia_N9 = "Nokia N9"
    Nokia_Lumia_520 = "Nokia Lumia 520"
    Microsoft_Lumia_550 = "Microsoft Lumia 550"
    Microsoft_Lumia_950 = "Microsoft Lumia 950"
    Galaxy_S3 = "Galaxy S3"
    Galaxy_S_III = "Galaxy S III"
    Galaxy_S5 = "Galaxy S5"
    Kindle_Fire_HDX = "Kindle Fire HDX"
    iPad_Mini = "iPad Mini"
    iPad = "iPad"
    iPad_Pro = "iPad Pro"
    iPad_8th_Gen = "iPad 8th Gen"
    iPad_10th_Gen = "iPad 10th Gen"
    iPad_Mini_4th_Gen = "iPad Mini 4th Gen"
    iPad_Mini_6th_Gen = "iPad Mini 6th Gen"
    Blackberry_PlayBook = "Blackberry PlayBook"
    Nexus_10 = "Nexus 10"
    Nexus_7 = "Nexus 7"
    Galaxy_Note_3 = "Galaxy Note 3"
    Galaxy_Note_II = "Galaxy Note II"
    Galaxy_Note_2 = "Galaxy Note 2"
    Galaxy_S20 = "Galaxy S20"
    Galaxy_S22 = "Galaxy S22"
    Galaxy_S21 = "Galaxy S21"
    Galaxy_S21_Ultra = "Galaxy S21 Ultra"
    Galaxy_S22_Ultra = "Galaxy S22 Ultra"
    Laptop_with_touch = "Laptop with touch"
    Laptop_with_HiDPI_screen = "Laptop with HiDPI screen"
    Laptop_with_MDPI_screen = "Laptop with MDPI screen"
    iPhone_XR = "iPhone XR"
    iPhone_XS_Max = "iPhone XS Max"
    iPhone_XS = "iPhone XS"
    Samsung_Galaxy_A5 = "Samsung Galaxy A5"
    Galaxy_A5 = "Galaxy A5"
    Samsung_Galaxy_S8 = "Samsung Galaxy S8"
    Galaxy_S8 = "Galaxy S8"
    LG_G6 = "LG G6"
    iPad_Air_2 = "iPad Air 2"
    iPad_6th_Gen = "iPad 6th Gen"
    iPhone_11 = "iPhone 11"
    iPhone_11_Pro_Max = "iPhone 11 Pro Max"
    iPhone_11_Pro = "iPhone 11 Pro"
    Galaxy_S10 = "Galaxy S10"
    Galaxy_S9_Plus = "Galaxy S9 Plus"
    Galaxy_S9 = "Galaxy S9"
    Galaxy_S10_Plus = "Galaxy S10 Plus"
    Galaxy_S8_Plus = "Galaxy S8 Plus"
    Galaxy_Note_10 = "Galaxy Note 10"
    Galaxy_Note_10_Plus = "Galaxy Note 10 Plus"
    Galaxy_Note_9 = "Galaxy Note 9"
    Galaxy_Note_8 = "Galaxy Note 8"
    Galaxy_Note_4 = "Galaxy Note 4"
    Galaxy_Tab_A_SM_T510 = "Galaxy Tab A SM-T510"
    Galaxy_Tab_A7_SM_T500 = "Galaxy Tab A7 SM-T500"
    Galaxy_Tab_S8 = "Galaxy Tab S8"
    Galaxy_Tab_A8 = "Galaxy Tab A8"
    Galaxy_S23 = "Galaxy S23"
    Galaxy_S23_Ultra = "Galaxy S23 Ultra"
    Galaxy_S25 = "Galaxy S25"
    Galaxy_S25_Ultra = "Galaxy S25 Ultra"
    Galaxy_A52s = "Galaxy A52s"
    Pixel_3 = "Pixel 3"
    Pixel_3_XL = "Pixel 3 XL"
    Pixel_4 = "Pixel 4"
    Pixel_4_XL = "Pixel 4 XL"
    Pixel_5 = "Pixel 5"
    Pixel_9 = "Pixel 9"
    iPad_7th_Gen = "iPad 7th Gen"
    OnePlus_7T = "OnePlus 7T"
    OnePlus_7T_Pro = "OnePlus 7T Pro"
    Galaxy_Tab_S7 = "Galaxy Tab S7"
    Sony_Xperia_10_II = "Sony Xperia 10 II"
    Huawei_Mate_50_Pro = "Huawei Mate 50 Pro"
    Huawei_Matepad_11 = "Huawei Matepad 11"
    Huawei_P30 = "Huawei P30"
    Xiaomi_Mi_A3 = "Xiaomi Mi A3"
    Huawei_P40 = "Huawei P40"
    Xiaomi_Redmi_Note_8 = "Xiaomi Redmi Note 8"
    Xiaomi_Mi_Poco_X3_Pro = "Xiaomi Mi Poco X3 Pro"
    Xiaomi_Poco_X3 = "Xiaomi Poco X3"
    iPhone_5_SE = "iPhone 5/SE"
    Pixel_7 = "Pixel 7"
    iPhone_15 = "iPhone 15"

    @DeprecatedEnumVariant
    def iPhone_5SE(self):
        """`iPhone_5SE` is deprecated. Use `iPhone_5_SE` instead"""
        return self.iPhone_5_SE

    @DeprecatedEnumVariant
    def iPhone_5S_E(self):
        """`iPhone_5S_E` is deprecated. Use `iPhone_5SE` instead"""
        return self.iPhone_5SE
