from __future__ import absolute_import, division, print_function, unicode_literals

from ..common import deprecated
from ..common.locators import VisualLocator, VisualLocatorSettings

deprecated.module(__name__)
__all__ = "VisualLocator", "VisualLocatorSettings"
